=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2011-05-17 09:04:56 +0000
+++ bzrlib/builtins.py	2011-05-17 12:12:09 +0000
@@ -5366,7 +5366,7 @@
         ]
 
     def run(self, submit_branch=None, public_branch=None, no_bundle=False,
-            no_patch=False, revision=None, remember=False, output=None,
+            no_patch=False, revision=None, remember=None, output=None,
             format=None, mail_to=None, message=None, body=None,
             strict=None, **kwargs):
         from bzrlib.send import send

=== modified file 'bzrlib/send.py'
--- bzrlib/send.py	2010-04-22 14:18:17 +0000
+++ bzrlib/send.py	2011-05-17 12:12:09 +0000
@@ -63,7 +63,8 @@
             submit_branch = stored_submit_branch
             remembered_submit_branch = "submit"
         else:
-            if stored_submit_branch is None or remember:
+            # Remembers if asked explicitly or no previous location is set
+            if remember or (remember is None and stored_submit_branch is None):
                 branch.set_submit_branch(submit_branch)
         if submit_branch is None:
             submit_branch = branch.get_parent()
@@ -95,7 +96,9 @@
         stored_public_branch = branch.get_public_branch()
         if public_branch is None:
             public_branch = stored_public_branch
-        elif stored_public_branch is None or remember:
+        # Remembers if asked explicitly or no previous location is set
+        elif (remember
+              or (remember is None and stored_public_branch is None)):
             branch.set_public_branch(public_branch)
         if no_bundle and public_branch is None:
             raise errors.BzrCommandError('No public branch specified or'

=== modified file 'bzrlib/tests/blackbox/test_send.py'
--- bzrlib/tests/blackbox/test_send.py	2011-01-10 22:20:12 +0000
+++ bzrlib/tests/blackbox/test_send.py	2011-05-17 12:12:09 +0000
@@ -346,7 +346,7 @@
 
 
 class TestSendStrictWithChanges(tests.TestCaseWithTransport,
-                                   TestSendStrictMixin):
+                                TestSendStrictMixin):
 
     # These are textually the same as test_push.strict_push_change_scenarios,
     # but since the functions are reimplemented here, the definitions are left
@@ -433,3 +433,80 @@
 class TestBundleStrictWithoutChanges(TestSendStrictWithoutChanges):
 
     _default_command = ['bundle-revisions', '../parent']
+
+
+# * first use remembers by default
+# * first use remembers if --remember is used
+# * first use does not remember if --no-remember is used
+
+# * default does not override stored
+# * --remember overrides stored
+# * --no-remember does not override stored
+
+class TestRemember(tests.TestCaseWithTransport, TestSendMixin):
+
+    _default_wd = 'work'
+
+    def setUp(self):
+        super(TestRemember, self).setUp()
+        grand_parent_tree = bzrdir.BzrDir.create_standalone_workingtree(
+            'grand_parent')
+        self.build_tree_contents([('grand_parent/file1', 'grand_parent')])
+        grand_parent_tree.add('file1')
+        grand_parent_tree.commit('initial commit', rev_id='grand_parent')
+        self.grand_parent_tree = grand_parent_tree
+
+        parent_bzrdir = grand_parent_tree.bzrdir.sprout('parent')
+        parent_tree = parent_bzrdir.open_workingtree()
+        parent_tree.commit('next commit', rev_id='parent')
+        self.parent_tree = parent_tree
+
+        branch_tree = parent_tree.bzrdir.sprout('branch').open_workingtree()
+        self.build_tree_contents([('branch/file1', 'branch')])
+        branch_tree.commit('last commit', rev_id='branch')
+        self.branch_tree = branch_tree
+
+    def prepare_next_uses(self):
+        # Do a first send that remembers the locations
+        self.do_send(['../parent', '../grand_parent'])
+        # Now create some new targets
+        new_grand_parent_tree = self.grand_parent_tree.bzrdir.sprout(
+            'new_grand_parent')
+        new_parent_tree = self.parent_tree.bzrdir.sprout('new_parent')
+
+    def assertLocations(self, expected_submit_branch, expected_public_branch):
+        br = self.branch_tree.branch
+        self.assertEquals(expected_submit_branch, br.get_submit_branch())
+        self.assertEquals(expected_public_branch, br.get_public_branch())
+
+    def do_send(self, *args):
+        # We always expect the same result here and care only about the
+        # arguments used and their consequences on the remembered locations
+        self.assertBundleContains(['branch'], *args)
+
+    def test_first_use_no_option(self):
+        self.do_send(['../parent', '../grand_parent'])
+        self.assertLocations('../parent', '../grand_parent')
+
+    def test_first_use_remember(self):
+        self.do_send(['--remember', '../parent', '../grand_parent'])
+        self.assertLocations('../parent', '../grand_parent')
+
+    def test_first_use_no_remember(self):
+        self.do_send(['--no-remember', '../parent', '../grand_parent'])
+        self.assertLocations(None, None)
+
+    def test_next_uses_no_option(self):
+        self.prepare_next_uses()
+        self.do_send(['../new_parent', '../new_grand_parent'])
+        self.assertLocations('../parent', '../grand_parent')
+
+    def test_next_uses_remember(self):
+        self.prepare_next_uses()
+        self.do_send(['--remember', '../new_parent', '../new_grand_parent'])
+        self.assertLocations('../new_parent', '../new_grand_parent')
+
+    def test_next_uses_no_remember(self):
+        self.prepare_next_uses()
+        self.do_send(['--no-remember', '../new_parent', '../new_grand_parent'])
+        self.assertLocations('../parent', '../grand_parent')

